import "./style.css";
//@ts-ignore
import init, { main } from "george";

async function run() {
  await init();
  main();
}

run();

async function getManpage(command: string) {
  const res = await fetch(`https://man.augustkline.com/${command}`, {
    method: "GET",
    credentials: "same-origin",
  });
  const parser = new DOMParser();
  const json = await res.json();
  const html = window.atob(json["html"]);
  const doc = parser.parseFromString(html, "text/html");
  return doc;
}

const sendInput = async (event: SubmitEvent) => {
  event.preventDefault();
  const form: HTMLFormElement = document.getElementById(
    "man-input",
  )! as HTMLFormElement;

  const input = form.elements[0] as HTMLInputElement;
  const command_text = input.value;
  input.value = "";
  const command = command_text.slice(4, command_text.length);
  try {
    await makeManpage(command);
    makeEcho(command_text, false);
  } catch (error) {
    makeEcho(command_text, true);
  }
};

const makeEcho = (command: string, error: boolean) => {
  const results = document.getElementById("results")!;
  const echo = document.createElement("div");
  const commandEl = document.createElement("code");
  commandEl.innerHTML = command;
  const result = document.createElement("p");
  if (error) {
    if (command.slice(0, 4) == "man ") {
      result.innerHTML = `Unrecognized command <code>${command.slice(4, command.length)}</code>`;
    } else {
      result.innerHTML = `Unrecognized command <code>${command}</code>`;
    }
    echo.className += " error";
  } else {
    result.innerHTML = `Opening man page for <code>${command.slice(4, command.length)}</code>`;
  }
  echo.append(commandEl, result);
  results.append(echo);
  fadeIn(echo);
};

const fadeIn = (el: HTMLElement) => {
  el.style.animation = "fade-in 0.3s";
};

const fadeInFlex = (el: HTMLElement) => {
  el.style.animation = "fade-in 0.5s, flex-in 0.8s";
};

const makeManpage = async (command: string) => {
  const demo = document.getElementById("man-demo")!;
  const doc = await getManpage(command);
  if (demo.children.length > 1) {
    const h1 = document.createElement("h1");
    const desc = doc.getElementsByClassName("Pp")!.item(0)?.innerHTML;
    const widget = document.getElementById("manpage-widget")!;
    widget.scrollTop = 0;
    h1.innerHTML = `${desc}`;
    widget.innerHTML = "";
    widget.style.setProperty("opacity", "1");
    widget.append(h1, ...doc.body.children);
  } else {
    const h1 = document.createElement("h1");
    const desc = doc.getElementsByClassName("Pp")!.item(0)?.innerHTML;
    const widget = document.createElement("div");
    widget.id = "manpage-widget";
    const container = document.createElement("div");
    container.id = "manpage-container";
    container.className = "widget";
    h1.innerHTML = `${desc}`;
    widget.innerHTML = "";
    widget.style.setProperty("opacity", "1");
    widget.append(h1, ...doc.body.children);
    container.append(widget);
    demo.append(container);
    fadeInFlex(container);
  }
};

const typeAnim = () => {
  let i = 0;
  const txt = '"here\'s what I bring to the team"';
  const speed = 75;
  const type = () => {
    if (i < txt.length) {
      document.getElementById("typed")!.innerHTML += txt.charAt(i);
      i++;
      setTimeout(type, speed);
    }
  };
  type();
};
setTimeout(typeAnim, 1000);

const manTerm = document.getElementById("man-terminal")!;
const input = document.getElementById("man-input")! as HTMLFormElement;
manTerm.addEventListener("click", () => {
  let prompt = input.children[1] as HTMLInputElement;
  prompt.focus();
});

const setTabIndices = () => {
  const canvases = document.getElementsByTagName("canvas")!;
  for (let canvas in canvases) {
    canvases[canvas].setAttribute("tabIndex", "-1");
  }
};

window.addEventListener("load", () => {
  // Wait for wasm to add canvas before changing indices
  setTimeout(setTabIndices, 1000);
});

input!.addEventListener("submit", sendInput);

const memoize = (fn) => {
  let cache = {};
  return (...args) => {
    let n = args[0];
    if (n in cache) {
      return cache[n];
    } else {
      let result = fn(n);
      cache[n] = result;
      return result;
    }
  };
};

const getAboutWords = (): {
  abouts: HTMLCollectionOf<HTMLAnchorElement>;
  displays: HTMLCollectionOf<HTMLDivElement>;
} => {
  let about = document.getElementById("about")!;
  let abouts = about.getElementsByTagName(
    "a",
  )! as HTMLCollectionOf<HTMLAnchorElement>;
  let aboutDisplay = document.getElementById("about-display")!;
  let displays = aboutDisplay.getElementsByClassName(
    "about",
  )! as HTMLCollectionOf<HTMLDivElement>;
  return { abouts, displays };
};

const setAboutWordsMouseEnter = () => {
  let { abouts, displays } = getAboutWords();
  for (let i = 0; i < abouts.length; i++) {
    let word = abouts.item(i)!;
    word.onmouseenter = () => {
      for (let j = 0; j < abouts.length; j++) {
        let display = displays.item(j)!;
        if (word != abouts.item(j)) {
          abouts.item(j)!.className = "";
          if (memoize(window.getComputedStyle(display)).opacity === "1") {
            display.style.animation = "fade-out 0.3s forwards";
          } else {
            display.style.opacity = "0";
            display.style.animation = "";
          }
          display.style.pointerEvents = "none";
        }
      }
      word.className = "august-accent-underline";
      displays.item(i)!.style.animation = "fade-in 0.3s forwards";
      displays.item(i)!.style.pointerEvents = "all";
    };
    word.onclick = (ev) => {
      ev.preventDefault();
    };
  }
};

const removeListenersMobile = () => {
  let { abouts } = getAboutWords();
  for (let i = 0; i < abouts.length; i++) {
    let word = abouts.item(i)!;
    word.onmouseenter = null;
    word.onclick = null;
  }
};

const onResize = () => {
  const mobile = window.matchMedia("all and (max-width: 70rem)").matches;
  if (mobile) {
    removeListenersMobile();
    setMobileAboutStyle();
  } else {
    setAboutWordsMouseEnter();
  }
};

window.addEventListener("resize", onResize);

const setMobileAboutStyle = () => {
  const { abouts, displays } = getAboutWords();
  for (let i = 0; i < abouts.length; i++) {
    abouts[i].className = "";
    displays[i].style.pointerEvents = "all";
  }
};

const onLoad = () => {
  const mobile = window.matchMedia("all and (max-width: 70rem)").matches;
  if (!mobile) {
    setAboutWordsMouseEnter();
  } else {
    setMobileAboutStyle();
  }
};

window.addEventListener("load", onLoad);
